# -*- coding: utf-8 -*-
"""
Created on Wed Mar 26 ‏‎12:11:48 2014
@author: Daniel Treceño 

It reads the upload file and returns a dictionary with the file data.

Input:
------
file_id -> django upload file

Output:
-------
Output -> { Header: File Header
            Mode: The mode value indicates which features were active during
                    data collection for the file. (ActiLife 5-User’s Manual - 
                    Table 17. http://dl.theactigraph.com/ActiLife5-PUB10DOC10-H.pdf
                    or ActiLife 6-User’s Manual - Table 6. http://dl.theactigraph.com/SFT12DOC13%20-%20ActiLife%206%20Users%20Manual%20(Rev%20A).pdf)
                    This function only support Mode = 45
            Epoch: list --> [hours, minutes, seconds]
            Start_time: list --> [0, hours, minutes, seconds]
            End_time: Start_time+Total_time
                    list --> [days, hours, minutes, seconds]
            Start_date: list --> [day, month, year]
            N_channels: return channels number
            Channels_names: channels name, list --> [name1, name2, ...]
            Channels_data: Channels[channel number][channel data]
            Total_time: Time was active the actimetry sensor 
                    list --> [days, hours, minutes, seconds]
            Time_array: time vector in seconds}

To add new mode:
----------------
Example:
elif Mode == 45:
    nChannels = 5 #number of channels in file .dat
    Channels_names = ['x Axis', 'y Axis', 'z Axis', 'Inclinometer', 'Pedometer', 'Vector Module'] #names ordered
    VarVectorModule = 3 #number of vector to do the vectorial sum
    N_channels = 6  #number of channels returned
    
Algorithm:
----------
-Open file
    -Read file. It separates the header from the data array
-Close file
-Find mode. It finds the mode value in the header
-Find epoch period. It finds the epoch period value in the header
-Find start time
-Find start date
-Select mode. It get the features of the mode
-Sort channels. With the features of the select mode, it sorts the channels data.
-Calculate total time.
-Calculate time array.

"""
import numpy

def read_file_dat(file_id):
    file_data = open(file_id)
    #file_data = file_id
    #file_data =file_name.read()
    #read file
    
    cont=0
    Header=''
    DataArray=[]
    for line in file_data:
        if (cont >= 10):
            numero=numpy.fromstring(line,dtype=int, sep=' ')
            DataArray=numpy.concatenate((DataArray, numero))
        else:
            Header=Header+line
        cont = cont + 1
        
    file_data.close()
    #Read header information
    #Mode
    IndexBeg=Header.index('Mode = ')+7
    IndexEnd=IndexBeg+3
    ModeString = Header[IndexBeg:IndexEnd]
    Mode = int(ModeString)
    #Epoch
    IndexBeg=Header.index('Epoch Period (hh:mm:ss) ')+24
    IndexEnd=IndexBeg+8
    EpochString = Header[IndexBeg:IndexEnd]
    Epoch = numpy.fromstring(EpochString,dtype=int, sep=':')
    #Start Time 15:00:00
    IndexBeg=Header.index('Start Time ')+11
    IndexEnd=IndexBeg+8
    TimeString = Header[IndexBeg:IndexEnd]
    Time = numpy.fromstring(TimeString,dtype=int, sep=':')
    #Start Date 19/05/2010
    IndexBeg=Header.index('Start Date ')+11
    IndexEnd=IndexBeg+10
    DateString = Header[IndexBeg:IndexEnd]
    Date = numpy.fromstring(DateString,dtype=int, sep='/')
    #Select mode
    if Mode == 45:
        nChannels = 5
        Channels_names = ['x Axis', 'y Axis', 'z Axis', 'Pedometer', 'Inclinometer',  'Vector Module']
        VarVectorModule = 3 #
        N_channels = 6
    if Mode == 44:
        nChannels = 4
        Channels_names = ['x Axis', 'y Axis', 'z Axis', 'Inclinometer', 'Vector Module']
        VarVectorModule = 3 #
        N_channels = 5
    #Read channels
    if VarVectorModule > 0:
        Channel = numpy.zeros((nChannels+1, len(DataArray)/nChannels), dtype=numpy.float)
        #vectors..
        for i in range(0,len(DataArray),nChannels):
            for j in range(0,nChannels):
                try:
                    Channel[j,(i+nChannels)/nChannels-1] = DataArray[i+j]
                except:
                    pass
        #vector module       
        for i in range(0, len(DataArray)/nChannels):
            num = 0
            for j in range (0, VarVectorModule):
                num = num + Channel[j,i]**2
            #Channel[nChannels-1+1,i] = numpy.sqrt(num)
            Channel[nChannels,i] = numpy.sqrt(num)
    else:        
        Channel = numpy.zeros((nChannels, len(DataArray)/nChannels), dtype=numpy.int)   
        for i in range(0,len(DataArray),nChannels):
            for j in range(0,nChannels):
                try:
                    Channel[j,(i+nChannels)/nChannels-1] = DataArray[i+j]
                except:
                    pass
                
    #Total time
    Ts = int(Epoch[0]*3600+Epoch[1]*60+Epoch[2])
    total = Channel[0].__len__()*Ts
    days = int(total/86400)
    hours = int((total-days*86400)/3600)  
    minutes = int((total-days*86400-hours*3600)/60)  
    seconds = total-days*86400-hours*3600-minutes*60 
    total_time = [days, hours, minutes, seconds]
    
    #Time Array
    time_array=range(0, total, Ts)

    #End Time
    start_time = Time.tolist()
    end_time = [0,0,0,0]
    end_time[3] = int((start_time[2]+total_time[3])%60)
    min_sec = int((start_time[2]+total_time[3])/60)
    end_time[2] = int((start_time[1]+total_time[2]+min_sec)%60)
    h_min   = int((start_time[1]+total_time[2]+min_sec)/60)
    end_time[1] = int((start_time[0]+total_time[1]+h_min)%24)
    d_h     = int((start_time[0]+total_time[1]+h_min)/24)
    end_time[0] = int(total_time[0]+d_h)

    #Return data
    Channel = Channel.astype(int)
    Output = {'Header':Header,
              'Mode':Mode,
              'Epoch':Epoch.tolist(),
              'Start_time':[0,int(start_time[0]),int(start_time[1]),int(start_time[2])],
              'End_time':end_time,
              'Start_date':Date.astype(int).tolist(),
              'N_channels':N_channels,
              'Channels_names':Channels_names,
              'Channels_data':Channel.tolist(),
              'Total_time':total_time,
              'Time_array':time_array}
    return  Output

#Test
#FileName = "382097060410.dat"
#Output = read_file_dat(FileName)
